@@ -75,6 +75,13 @@ where
         BBox { min: p, max: p }
     }
     /// Constructs the smallest bounding box containing two points.
+    #[deprecated = "Use `from_corners` instead."]
+    pub fn from_points(p0: Point<S, V>, p1: Point<S, V>) -> BBox<S, V> {
+        let min = p0.min(p1);
+        let max = p0.max(p1);
+        BBox { min, max }
+    }
+    /// Constructs the smallest bounding box containing two points.
     ///
     /// # Example
     /// ```
@@ -83,13 +90,41 @@ where
     /// # use lowdim::BBox2d;
     /// let p0 = p2d(2, 3);
     /// let p1 = p2d(-1, 4);
-    /// assert_eq!(bb2d(-1..3, 3..5), BBox2d::from_points(p0, p1));
+    /// assert_eq!(bb2d(-1..3, 3..5), BBox2d::from_corners(p0, p1));
     /// ```
-    pub fn from_points(p0: Point<S, V>, p1: Point<S, V>) -> BBox<S, V> {
+    pub fn from_corners(p0: Point<S, V>, p1: Point<S, V>) -> BBox<S, V> {
         let min = p0.min(p1);
         let max = p0.max(p1);
         BBox { min, max }
     }
+    /// Constructs the minimal bounding box containing points from an iterator,
+    /// or `None` if the iterator is empty.
+    ///
+    /// # Example
+    /// ```
+    /// # use lowdim::p2d;
+    /// # use lowdim::bb2d;
+    /// # use lowdim::BBox2d;
+    /// let points = vec![p2d(2, 3), p2d(-1, 4), p2d(1, 0)];
+    /// assert_eq!(Some(bb2d(-1..=2, 0..=4)), BBox2d::enclosing(&points));
+    /// ```
+    pub fn enclosing<'a, II>(ii: II) -> Option<BBox<S, V>>
+    where
+        S: 'a,
+        V: 'a,
+        II: IntoIterator<Item = &'a Point<S, V>>,
+    {
+        let mut iter = ii.into_iter();
+        if let Some(p0) = iter.next() {
+            let mut bbox = BBox::from_point(*p0);
+            for p in iter {
+                bbox = bbox.extend_to(*p);
+            }
+            Some(bbox)
+        } else {
+            None
+        }
+    }
     /// The minimal point in the bounding box.
     ///
     /// # Example
@@ -360,6 +395,7 @@ where
         Bound::Excluded(s) => *s - S::one(),
         Bound::Unbounded => panic!("unbounded ranges are not allowed"),
     };
+    assert!(start <= end);
     start..=end
 }
 
@@ -372,9 +408,9 @@ where
 {
     let rx = to_range_inclusive(rx);
     let ry = to_range_inclusive(ry);
-    let p_min = p2d(*rx.start(), *ry.start());
-    let p_max = p2d(*rx.end(), *ry.end());
-    BBox2d::from_points(p_min, p_max)
+    let min = p2d(*rx.start(), *ry.start());
+    let max = p2d(*rx.end(), *ry.end());
+    BBox { min, max }
 }
 /// Creates a 3d bounding box from ranges of x, y and z coordinates.
 pub fn bb3d<S, RX, RY, RZ>(rx: RX, ry: RY, rz: RZ) -> BBox3d<S>
@@ -387,9 +423,9 @@ where
     let rx = to_range_inclusive(rx);
     let ry = to_range_inclusive(ry);
     let rz = to_range_inclusive(rz);
-    let p_min = p3d(*rx.start(), *ry.start(), *rz.start());
-    let p_max = p3d(*rx.end(), *ry.end(), *rz.end());
-    BBox3d::from_points(p_min, p_max)
+    let min = p3d(*rx.start(), *ry.start(), *rz.start());
+    let max = p3d(*rx.end(), *ry.end(), *rz.end());
+    BBox { min, max }
 }
 /// Creates a 4d bounding box from ranges of x, y, z and w coordinates.
 pub fn bb4d<S, RX, RY, RZ, RW>(rx: RX, ry: RY, rz: RZ, rw: RW) -> BBox4d<S>
@@ -404,9 +440,9 @@ where
     let ry = to_range_inclusive(ry);
     let rz = to_range_inclusive(rz);
     let rw = to_range_inclusive(rw);
-    let p_min = p4d(*rx.start(), *ry.start(), *rz.start(), *rw.start());
-    let p_max = p4d(*rx.end(), *ry.end(), *rz.end(), *rw.end());
-    BBox4d::from_points(p_min, p_max)
+    let min = p4d(*rx.start(), *ry.start(), *rz.start(), *rw.start());
+    let max = p4d(*rx.end(), *ry.end(), *rz.end(), *rw.end());
+    BBox { min, max }
 }
 
 impl<S> BBox3d<S>
@@ -769,11 +805,29 @@ mod tests {
     }
 
     #[test]
+    #[allow(deprecated)]
     fn test_from_points() {
         let bb = BBox2d::from_points(p2d(-2, 3), p2d(4, 7));
         assert_eq!(bb2d(-2..=4, 3..=7), bb);
     }
 
+    #[test]
+    fn test_from_corners() {
+        let bb = BBox2d::from_corners(p2d(-2, 3), p2d(4, 7));
+        assert_eq!(bb2d(-2..=4, 3..=7), bb);
+    }
+
+    #[test]
+    fn test_enclosing() {
+        let bb = BBox2d::enclosing(&vec![p2d(-2, 3), p2d(4, 7)]);
+        assert_eq!(Some(bb2d(-2..=4, 3..=7)), bb);
+    }
+    #[test]
+    fn test_enclosing_empty() {
+        let bb = <BBox2d<i64>>::enclosing(&vec![]);
+        assert_eq!(None, bb);
+    }
+
     #[test]
     fn test_min_2d() {
         let bb = bb2d(-2..=2, -1..=1);
@@ -1246,7 +1300,7 @@ mod tests {
     }
 
     #[test]
-    fn test_exact_size_iterator_2d() {
+    fn test_exact_size_iterator_2d_owned() {
         let bb = bb2d(1..=2, -1..=0);
         let mut it = bb.into_iter();
         assert_eq!((4, Some(4)), it.size_hint());
@@ -1257,7 +1311,18 @@ mod tests {
         assert_eq!(3, it.len());
     }
     #[test]
-    fn test_exact_size_iterator_3d() {
+    fn test_exact_size_iterator_2d_ref() {
+        let bb = &bb2d(1..=2, -1..=0);
+        let mut it = bb.into_iter();
+        assert_eq!((4, Some(4)), it.size_hint());
+        assert_eq!(4, it.len());
+        let p = it.next();
+        assert_eq!(Some(p2d(1, -1)), p);
+        assert_eq!((3, Some(3)), it.size_hint());
+        assert_eq!(3, it.len());
+    }
+    #[test]
+    fn test_exact_size_iterator_3d_owned() {
         let bb = bb3d(1..=2, -1..=0, -2..=-1);
         let mut it = bb.into_iter();
         assert_eq!((8, Some(8)), it.size_hint());
@@ -1268,7 +1333,18 @@ mod tests {
         assert_eq!(7, it.len());
     }
     #[test]
-    fn test_exact_size_iterator_4d() {
+    fn test_exact_size_iterator_3d_ref() {
+        let bb = &bb3d(1..=2, -1..=0, -2..=-1);
+        let mut it = bb.into_iter();
+        assert_eq!((8, Some(8)), it.size_hint());
+        assert_eq!(8, it.len());
+        let p = it.next();
+        assert_eq!(Some(p3d(1, -1, -2)), p);
+        assert_eq!((7, Some(7)), it.size_hint());
+        assert_eq!(7, it.len());
+    }
+    #[test]
+    fn test_exact_size_iterator_4d_owned() {
         let bb = bb4d(1..=2, -1..=0, -2..=-1, 3..=4);
         let mut it = bb.into_iter();
         assert_eq!((16, Some(16)), it.size_hint());
@@ -1278,4 +1354,15 @@ mod tests {
         assert_eq!((15, Some(15)), it.size_hint());
         assert_eq!(15, it.len());
     }
+    #[test]
+    fn test_exact_size_iterator_4d_ref() {
+        let bb = &bb4d(1..=2, -1..=0, -2..=-1, 3..=4);
+        let mut it = bb.into_iter();
+        assert_eq!((16, Some(16)), it.size_hint());
+        assert_eq!(16, it.len());
+        let p = it.next();
+        assert_eq!(Some(p4d(1, -1, -2, 3)), p);
+        assert_eq!((15, Some(15)), it.size_hint());
+        assert_eq!(15, it.len());
+    }
 }
